clear all;
close all;

% Mega test for dvbsenco


% Simulation parms
B3db = 56000000;
p_opt = ' -p 1/2 '; % puncturing rate
I = 4; % upsampling rate

Fs = I * B3db / 1.28;
Ts = 1/Fs;

% Run the transmission side
dos(['dvbsenco -i test.m2v -o test.bin -d ercvpmt -v 2 -c rand ' p_opt ' -I ' int2str(I) 'x']);

% First the MPEG-2 file
fid = fopen('test.m2v', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' MpegTs.']);
figure(1);
clf;
plot(A(1:1000));
title('MPEG-2 TS');

% Then the RS file
fid = fopen('test_rs.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' RS.']);
figure(2);
clf;
plot(A(1:1000));
title('RS codewords');

% Then the ConvInt file
fid = fopen('test_ci.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' ConvInt.']);
figure(3);
clf;
plot(A(1:1000));
title('Convolutional interleaved bytes');

% Then the ConvEnco file
fid = fopen('test_ve.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' ConvEnco.']);
figure(4);
clf;
plot(A(1:1000));
title('Convolutional encoded bits');

% Then the Punct file
fid = fopen('test_pu.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' Punct.']);
figure(5);
clf;
plot(A(1:1000));
title('Punctured bits');

% Then the Map file
fid = fopen('test_ma.bin', 'r');
[A, len] = fread(fid, inf, 'float32');
fclose(fid);
B = A(1:2:end) + j.*A(2:2:end);
disp(['Read ' num2str(length(B)) ' Map.']);
figure(8);
clf;
plot(B(1:1000), 'o');
title('Scatter plot of data carriers');
xlabel('Real part');
ylabel('Imaginary part');
figure(9);
clf;
subplot(211);
plot(real(B(1:1000)));
title('Mapped symbols');
xlabel('Inphase');
subplot(212);
plot(imag(B(1:1000)));
xlabel('Quadrature');

% Then the TimeDom file
h = rcosfir(0.35, [-7.8 7.8], I, 1, 'sqrt'); %Receiving filter
fid = fopen('test.bin', 'r');
[A, len] = fread(fid, inf, 'float32');
fclose(fid);
B = A(1:2:240000) + j.*A(2:2:240000);
%B = A(1:2:end) + j.*A(2:2:end);
B = filter(h, 1, B);
powB = (B' * B) / length(B);
%B = B ./ sqrt(powB);
disp(['Read ' num2str(length(B)) ' TimeDom.']);
figure(12);
clf;
plot(B((1 + 62):I:30000), '.');
title('Scatter plot of time domain samples');
xlabel('Real part');
ylabel('Imaginary part');
figure(13);
clf;
subplot(211);
plot(Ts * [0:29999] * 1e6, real(B(1:30000)));
title('Time domain')
ylabel('Inphase')
xlabel('Time (\mus)');
subplot(212);
plot(Ts * [0:29999] * 1e6, imag(B(1:30000)));
ylabel('Quadrature')
xlabel('Time (\mus)');
[Pxx, f] = psd(B, 512, Fs / 1000000, boxcar(512), 32);
figure(14);
clf;
plot(f - Fs/2000000, 10*log10(fftshift(Pxx)));
grid on;
title('Power Spectrum Density of \itdvbsenco\rm output');
xlabel('Frequency (MHz)');
ylabel('PSD (dB_c/Hz)');
figure(15);
clf;
subplot(211);
[prob, dom] = hist(real(B), 100);
plot(dom, prob / length(B));
title('Probability distribution function for I and Q components');
xlabel('I amplitude');
ylabel('%');
subplot(212);
[prob, dom] = hist(imag(B), 100);
plot(dom, prob / length(B), 'b-');
xlabel('Q amplitude');
ylabel('%');
